<h3><%= @preview.preview_name.titleize %></h3>
<ul>
  <% @preview.examples.each do |example| %>
    <li><%= link_to example, preview_view_component_path("#{@preview.preview_name}/#{example}") %></li>
  <% end %>
</ul>
