<div>
  <% if header? %>
    <div id="header"><%= header %></div>
  <% end %>
  <% items.each do |item| %>
    <%= item %>
  <% end %>
  <% fields.each do |field| %>
    <%= field %>
  <% end %>
</div>
