<%= render(SlotsComponent.new) do |component| %>
  <% component.with_title do %>
    <p>This is my title!</p>
  <% end %>
  <% component.with_subtitle do %>
    <small>This is my subtitle!</small>
  <% end %>

  <% component.with_tab do %>
    <h1>Tab A</h1>
  <% end %>
  <% component.with_tab do %>
    <h3>Tab B</h3>
  <% end %>

  <% component.with_item do %>
    <h1>Item A</h1>
  <% end %>
  <% component.with_item(highlighted: true) do %>
    <h3>Item B</h3>
  <% end %>
  <% component.with_item do %>
    <h2>Item C</h2>
  <% end %>

  <% component.with_footer(classes: "text-blue") do %>
    <h3>This is the footer</h3>
  <% end %>
<% end %>
