<%= render SectionComponent.new do |section| %>
  <% section.with_nav do |nav| %>
    <% nav.with_action_menu do |menu| %>
      <% menu.with_list do |list| %>
        <% list.with_item do %>
          <%= t(".deep_action") %>
        <% end %>
      <% end %>
    <% end %>
  <% end %>
<% end %>
