<h1>List</h1>

<div class="body">
  <% if items.any? %>
    <% items.each do |item| %>
      <div class="item <%= item.classes %>">
        <%= item %>
      </div>
    <% end %>
  <% else %>
    No items provided
  <% end %>
</div>
