---
layout: default
title: API reference
nav_order: 3
---

<!-- Warning: AUTO-GENERATED file, don't edit. Add code comments to your Ruby instead <3 -->

# API

<% @sections.each do |section| %>
## <%= section.heading %>

<% section.methods.each do |method| %>
### <%= render ViewComponent::DocsBuilderComponent::MethodDoc.new(method, section.show_types) %>

<% end %>
<% section.error_klasses.each do |error_klass| %>
### <%= render ViewComponent::DocsBuilderComponent::ErrorKlassDoc.new(error_klass, section.show_types) %>

<% end %>
<% end %>
