<figure data-index="<%= @iteration.index %>" class="<%= "first" if @iteration.first? %>">
  <img src="<%= @item.url %>" alt="<%= @item.title %>">
  <figcaption>
    Photo.<%= @counter %> - <%= @item.caption %>
  </figcaption>
<figure>
