<% if @render_args[:component] %>
  <%= render(@render_args[:component], @render_args[:args], &@render_args[:block]) %>
<% else %>
  <%= render template: @render_args[:template], locals: @render_args[:locals] || {} %>
<% end %>
